/*

  yyycpu - a virtual machine
  Copyright (C) 2000  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "hexconvert.hpp"

static const uint8 hextobin[256] = {
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,    // 0
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,    // 16
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,    // 32
  0,   1,   2,   3,   4,   5,   6,   7,   8,   9,   0,   0,   0,   0,   0,   0,    // 48
  0,  10,  11,  12,  13,  14,  15,   0,   0,   0,   0,   0,   0,   0,   0,   0,    // 64
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,    // 80
  0,  10,  11,  12,  13,  14,  15,   0,   0,   0,   0,   0,   0,   0,   0,   0,    // 96
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,    // 112
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,    // 128
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,    // 144
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,    // 160
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,    // 176
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,    // 192
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,    // 208
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,    // 224
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0     // 240
};

inline uint8 hex4touint8(const char hex)
{
  return hextobin[hex];
};

// non public
inline uint16 hex4touint16(const char hex)
{
  return hex4touint8(hex);
};

// non public
inline uint32 hex4touint32(const char hex)
{
  return hex4touint8(hex);
};

uint8 hex8touint8(const char *const hex)
{
  return (hex4touint8(hex[0])<<4)
    | hex4touint8(hex[1]);
};

uint16 hex12touint16(const char *const hex)
{
  return (hex4touint16(hex[0])<<8)
    | (hex4touint16(hex[1])<<4)
    | hex4touint16(hex[2]);
};

uint16 hex16touint16(const char *const hex)
{
  return (hex4touint16(hex[0])<<12)
    | (hex4touint16(hex[1])<<8)
    | (hex4touint16(hex[2])<<4)
    | hex4touint16(hex[3]);
};

uint32 hex32touint32(const char *const hex)
{
  return (hex4touint32(hex[0])<<28)
    | (hex4touint32(hex[1])<<24)
    | (hex4touint32(hex[2])<<20)
    | (hex4touint32(hex[3])<<16)
    | (hex4touint32(hex[4])<<12)
    | (hex4touint32(hex[5])<<8)
    | (hex4touint32(hex[6])<<4)
    | hex4touint32(hex[7]);
};


static const uint8 bintohex[16] = {
  '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
  'A', 'B', 'C', 'D', 'E', 'F' };

inline void uint8tohex4(const uint8 val, char *const hex)
{
  hex[0]=bintohex[val];
};

inline void uint8tohex8(const uint8 val, char *const hex)
{
  hex[0]=bintohex[val>>4];
  hex[1]=bintohex[val&15];
};

inline void uint16tohex12(const uint16 val, char *const hex)
{
  hex[0]=bintohex[val>>8];
  hex[1]=bintohex[(val>>4)&15];
  hex[2]=bintohex[val&15];
};

inline void uint16tohex16(const uint16 val, char *const hex)
{
  hex[0]=bintohex[val>>12];
  hex[1]=bintohex[(val>>8)&15];
  hex[2]=bintohex[(val>>4)&15];
  hex[3]=bintohex[val&15];
};

inline void uint32tohex32(const uint32 val, char *const hex)
{
  hex[0]=bintohex[val>>28];
  hex[1]=bintohex[(val>>24)&15];
  hex[2]=bintohex[(val>>20)&15];
  hex[3]=bintohex[(val>>16)&15];
  hex[4]=bintohex[(val>>12)&15];
  hex[5]=bintohex[(val>>8)&15];
  hex[6]=bintohex[(val>>4)&15];
  hex[7]=bintohex[val&15];
};


static char hexbuf4[] ="-";
static char hexbuf8[] ="--";
static char hexbuf12[]="---";
static char hexbuf16[]="----";
static char hexbuf32[]="--------";
char *const uint8toglobalhex4(const uint8 val)
{
  uint8tohex4(val,hexbuf4);
  return hexbuf4;
};

char *const uint8toglobalhex8(const uint8 val)
{
  uint8tohex8(val,hexbuf8);
  return hexbuf8;
};

char *const uint16toglobalhex12(const uint16 val)
{
  uint16tohex12(val,hexbuf12);
  return hexbuf12;
};

char *const uint16toglobalhex16(const uint16 val)
{
  uint16tohex16(val,hexbuf16);
  return hexbuf16;
};

char *const uint32toglobalhex32(const uint32 val)
{
  uint32tohex32(val,hexbuf32);
  return hexbuf32;
};
