/*

  yyycpu - a virtual machine
  Copyright (C) 2000  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef __HEXCONVERT_HPP
#define __HEXCONVERT_HPP

#include "binarydefs.hpp"

// hex string to uint conversion
// ----------------------
// hex buffer is always of the filled size! trailing zero unneeded!
// i.e. hex16touint16("FF") will segfault ! because "FF" is too short!
// no leading garbage allowed ! (i.e. " F" is forbidden !)
// all illegal characters count as 0
uint8 hex4touint8(const char hex);
uint8 hex8touint8(const char *const hex);
uint16 hex12touint16(const char *const hex);
uint16 hex16touint16(const char *const hex);
uint32 hex32touint32(const char *const hex);

// uint to hex string conversion
// -----------------------------
// hex buffer must be of correct size (or bigger) !
// trailing zero will NOT NOT NOT NOT NOT be created !!!
// if number doesn't fit, the least significant bits are used only.
void uint8tohex4(const uint8 val, char *const hex);
void uint8tohex8(const uint8 val, char *const hex);
void uint16tohex12(const uint16 val, char *const hex);
void uint16tohex16(const uint16 val, char *const hex);
void uint32tohex32(const uint32 val, char *const hex);

// uint to hex string conversion with a static global buffer
// ---------------------------------------------------------
// laws of above sections are valid here too.
// these functions all put data into their own global output buffer.
// the output is always a valid string with a terminating zero
// printf("%s %s\n", uint4toglobalhex4(3), uint4toglobalhex4(7))
// is NOT ok ! because the latter call overwrites the buffer
//   of the first call!
// but
// printf("%s %s\n", uint4toglobalhex4(3), uint8toglobalhex8(7))
// is OK, because both all functions use their own global buffer
// ---> these routines are of course NOT thread safe !
char *const uint8toglobalhex4(const uint8 val);
char *const uint8toglobalhex8(const uint8 val);
char *const uint16toglobalhex12(const uint16 val);
char *const uint16toglobalhex16(const uint16 val);
char *const uint32toglobalhex32(const uint32 val);

#endif
