/*

  yyycpu - a virtual machine
  Copyright (C) 2000  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "codemanager.hpp"

codemanager::codemanager()
{
};

codemanager::~codemanager()
{
};

const uint8 codemanager::get8(const uint32 address)
{
  return code[address];
};

void codemanager::put8(const uint32 address, const uint8 value)
{
  if (code.size()<=address) code.resize(address+1,0);
  code[address]=value;
};

const uint16 codemanager::get16(const uint32 address)
{
  uint16 r = uint16(code[address])<<8;
  r |= code[address+1];
  return r;
};

void codemanager::put16(const uint32 address, const uint16 value)
{
  if (code.size()<=address+1) code.resize(address+2,0);
  code[address]=value>>8;
  code[address+1]=value&255;
};

void codemanager::put32(const uint32 address, const uint32 value)
{
  if (code.size()<=address+3) code.resize(address+4,0);
  code[address]=value>>24;
  code[address+1]=(value>>16)&255;
  code[address+2]=(value>>8)&255;
  code[address+3]=value&255;
};

void codemanager::writeout(FILE *const output) const
{
  for (vector<uint8>::size_type i=0;i<code.size();i++)
    if (fputc(code[i],output)==EOF) throw "codemanager::writeout: fputc error";
};

uint32 codemanager::len() const
{
  return code.size();
};

void codemanager::cut (const uint32 address, const uint32 count)
{
  code.erase(&code[address],&code[address+count]);
};
