/*

  yyycpu - a virtual machine
  Copyright (C) 2000  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "constlist.hpp"

#include "global.hpp"

#include <stdio.h>

constlist::constlist()
{
};

constlist::index_t constlist::addADD (index_t c0, index_t c1)
{
  elem x;
  x.type=elem::ADD;
  x.child[0]=c0;
  x.child[1]=c1;
  x.valuedefined=false;
  vec.push_back(x);
  return vec.size()-1;
};

constlist::index_t constlist::addSUB (index_t c0, index_t c1)
{
  elem x;
  x.type=elem::SUB;
  x.child[0]=c0;
  x.child[1]=c1;
  x.valuedefined=false;
  vec.push_back(x);
  return vec.size()-1;
};

constlist::index_t constlist::addMUL (index_t c0, index_t c1)
{
  elem x;
  x.type=elem::MUL;
  x.child[0]=c0;
  x.child[1]=c1;
  x.valuedefined=false;
  vec.push_back(x);
  return vec.size()-1;
};

constlist::index_t constlist::addDIV (index_t c0, index_t c1)
{
  elem x;
  x.type=elem::DIV;
  x.child[0]=c0;
  x.child[1]=c1;
  x.valuedefined=false;
  vec.push_back(x);
  return vec.size()-1;
};

constlist::index_t constlist::addLAB (labellist::index_t lab)
{
  elem x;
  x.type=elem::LAB;
  x.valuedefined=false;
  x.labelindex=lab;
  vec.push_back(x);
  return vec.size()-1;
};

constlist::index_t constlist::addVAL (elem::value_t val)
{
  elem x;
  x.type=elem::VAL;
  x.value=val;
  x.valuedefined=true;
  vec.push_back(x);
  return vec.size()-1;
};

constlist::index_t constlist::addTOP5 (index_t c0)
{
  elem x;
  x.type=elem::TOP5;
  x.child[0]=c0;
  x.address=0;
  x.valuedefined=false;
  x.leftshift=0;
  vec.push_back(x);
  return vec.size()-1;
};

constlist::index_t constlist::addTOP8 (index_t c0)
{
  elem x;
  x.type=elem::TOP8;
  x.child[0]=c0;
  x.address=0;
  x.valuedefined=false;
  vec.push_back(x);
  return vec.size()-1;
};

constlist::index_t constlist::addTOP12 (index_t c0)
{
  elem x;
  x.type=elem::TOP12;
  x.child[0]=c0;
  x.address=0;
  x.valuedefined=false;
  x.leftshift=0;
  vec.push_back(x);
  return vec.size()-1;
};

constlist::index_t constlist::addTOP16 (index_t c0)
{
  elem x;
  x.type=elem::TOP16;
  x.child[0]=c0;
  x.address=0;
  x.valuedefined=false;
  vec.push_back(x);
  return vec.size()-1;
};

constlist::index_t constlist::addTOP32 (index_t c0)
{
  elem x;
  x.type=elem::TOP32;
  x.child[0]=c0;
  x.address=0;
  x.valuedefined=false;
  vec.push_back(x);
  return vec.size()-1;
};


void constlist::debugprint()
{
  printf("CONSTLIST DEBUG\n");
  for (index_t i=0;i<vec.size();++i) {
    printf("INDEX %d  ",i);
    switch (vec[i].type) {
    case constlist::elem::ADD: printf("ADD %d, %d",vec[i].child[0],vec[i].child[1]); break;
    case constlist::elem::SUB: printf("SUB %d, %d",vec[i].child[0],vec[i].child[1]); break;
    case constlist::elem::MUL: printf("MUL %d, %d",vec[i].child[0],vec[i].child[1]); break;
    case constlist::elem::DIV: printf("DIV %d, %d",vec[i].child[0],vec[i].child[1]); break;
    case constlist::elem::MOD: printf("MOD %d, %d",vec[i].child[0],vec[i].child[1]); break;
    case constlist::elem::SHL: printf("SHL %d, %d",vec[i].child[0],vec[i].child[1]); break;
    case constlist::elem::SHR: printf("SHR %d, %d",vec[i].child[0],vec[i].child[1]); break;
    case constlist::elem::VAL: printf("VAL"); break;
    case constlist::elem::LAB: printf("LAB %d", vec[i].labelindex); break;
    case constlist::elem::TOP5: printf("TOP5 %d [%d]",vec[i].child[0],vec[i].address); break;
    case constlist::elem::TOP8: printf("TOP8 %d [%d]",vec[i].child[0],vec[i].address); break;
    case constlist::elem::TOP12: printf("TOP12 %d [%d]",vec[i].child[0],vec[i].address); break;
    case constlist::elem::TOP16: printf("TOP16 %d [%d]",vec[i].child[0],vec[i].address); break;
    case constlist::elem::TOP32: printf("TOP32 %d [%d]",vec[i].child[0],vec[i].address); break;
    default:
      printf("unknown (to the debugprint routine) type\n");
    };
    if (vec[i].valuedefined) printf(" == [%lld]",vec[i].value); else printf(" == [undefined]");
    printf("\n");
  };
};

void constlist::fillin()
{
  for (index_t i=0;i<vec.size();++i) complete(i);
};

void constlist::complete(index_t i)
{
  elem &e = vec[i];
  if (e.valuedefined) return;
  switch (e.type) {
  case elem::ADD: complete(e.child[0]); complete(e.child[1]);
    e.value=vec[e.child[0]].value + vec[e.child[1]].value; e.valuedefined=true; break;
  case elem::SUB: complete(e.child[0]); complete(e.child[1]);
    e.value=vec[e.child[0]].value - vec[e.child[1]].value; e.valuedefined=true; break;
  case elem::MUL: complete(e.child[0]); complete(e.child[1]);
    e.value=vec[e.child[0]].value * vec[e.child[1]].value; e.valuedefined=true; break;
  case elem::DIV: complete(e.child[0]); complete(e.child[1]);
    e.value=vec[e.child[0]].value / vec[e.child[1]].value; e.valuedefined=true; break;
  case elem::LAB: e.value=ll->get(e.labelindex).address; e.valuedefined=true; break;
  case elem::TOP5:
    complete(e.child[0]); e.value=vec[e.child[0]].value;
    e.valuedefined=true;
    if ((e.value<-16) || (e.value>31)) throw "constlist::complete:5bit: value doesn't fit";
    cm->put16(e.address,cm->get16(e.address) | ((e.value&31)<<e.leftshift)); break;
  case elem::TOP8:
    complete(e.child[0]); e.value=vec[e.child[0]].value;
    e.valuedefined=true;
    if ((e.value<-128) || (e.value>255)) throw "constlist::complete:8bit: value doesn't fit";
    cm->put8(e.address,e.value); break;
  case elem::TOP12:
    complete(e.child[0]); e.value=vec[e.child[0]].value;
    e.valuedefined=true;
    if ((e.value<-2048) || (e.value>4095)) throw "constlist::complete:12bit: value doesn't fit";
    cm->put16(e.address,cm->get16(e.address) | ((e.value&4095)<<e.leftshift)); break;
  case elem::TOP16:
    complete(e.child[0]); e.value=vec[e.child[0]].value;
    e.valuedefined=true;
    if ((e.value<-32768) || (e.value>65535)) throw "constlist::complete:16bit: value doesn't fit";
    cm->put16(e.address,e.value); break;
  case elem::TOP32:
    complete(e.child[0]); e.value=vec[e.child[0]].value;
    e.valuedefined=true;
    if ((e.value<-2147483648ll) || (e.value>4294967295ll)) throw "constlist::complete:32bit: value doesn't fit";
    cm->put32(e.address,e.value); break;
  default: break;
  };
};
