/*

  yyycpu - a virtual machine
  Copyright (C) 2000  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef __CONSTLIST_HPP
#define __CONSTLIST_HPP

#include "labellist.hpp"

#include <vector>

class constlist {
public:
  constlist ();
  constlist (constlist &);
  constlist & operator= (const constlist &);

  struct elem;
  typedef vector<elem>::size_type index_t;

  struct elem {
    enum type_t {
      ADD, SUB, MUL, DIV, MOD, SHL, SHR, VAL, LAB,
      TOP5, TOP8, TOP12, TOP16, TOP32
    };
    type_t type;

    typedef int64 value_t;
    value_t value;
    bool valuedefined;
    uint8 leftshift;

    uint32 address;
    labellist::index_t labelindex;

    index_t child[2];
  };

  index_t addADD (index_t c0, index_t c1);
  index_t addSUB (index_t c0, index_t c1);
  index_t addMUL (index_t c0, index_t c1);
  index_t addDIV (index_t c0, index_t c1);
  index_t addVAL (elem::value_t val);
  index_t addLAB (labellist::index_t lab);
  index_t addTOP5 (index_t c0);
  index_t addTOP8 (index_t c0);
  index_t addTOP12 (index_t c0);
  index_t addTOP16 (index_t c0);
  index_t addTOP32 (index_t c0);

  inline elem & get (const index_t constindex) { return vec[constindex]; };

  void debugprint ();
  void fillin ();

private:
  vector<elem> vec;
  void complete(index_t i);
};

#endif
