/*

  yyycpu - a virtual machine
  Copyright (C) 2000  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "labellist.hpp"

#include "global.hpp"

#include <stdio.h>

labellist::labellist()
{
};

labellist::~labellist()
{
};

labellist::index_t labellist::findoraddundefined (const symbollist::index_t symbolindex)
{
  for (index_t i=0;i<vec.size();i++)
    if (vec[i].symbolindex==symbolindex) return i;
  vec.push_back(elem(symbolindex,0,false));
  return vec.size()-1;
};

void labellist::defineoradd (const symbollist::index_t symbolindex, const uint32 address)
{
  for (index_t i=0;i<vec.size();i++)
    if (vec[i].symbolindex==symbolindex) {
      if (vec[i].defined) throw "labellist::defineoradd: cannot define an already defined label";
      vec[i].defined=true;
      vec[i].address=address;
      return;
    };
  vec.push_back(elem(symbolindex,address,true));
};

void labellist::cut (const uint32 address, const uint32 count)
{
  uint32 firstmover=address+count;
  for (index_t i=0;i<vec.size();i++)
    if (vec[i].address>=firstmover) vec[i].address-=count;
};

void labellist::assertalldefined() const
{
  for (index_t i=0;i<vec.size();i++)
    if (!vec[i].defined) throw "labellist::assertalldefined: undefined label found!";
};

void labellist::debugprint()
{
  printf("LABELLIST DEBUG\n");
  for (index_t i=0;i<vec.size();i++) {
    printf("INDEX=%d SYMBOLINDEX=%d ADDRESS=", i, vec[i].symbolindex);
    if (vec[i].defined)
      printf("%d\n", vec[i].address);
    else
      printf("UNDEFINED\n");
  };
};
