/*

  yyycpu - a virtual machine
  Copyright (C) 2000  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <unistd.h>
#include <stdio.h>

#include "global.hpp"

void usage ()
{
  fprintf(stderr,"Usage: ycpuasm input output\n\n");
  exit(1);
};

void error (const char *const msg)
{
  fprintf(stderr,"Error: %s\n\n", msg);
  exit(1);
};

int main(int argc, char *argv[])
{
  try{
    if (argc!=3) usage();
  
    yyin = fopen(argv[1],"r");
    if (!yyin) error("Cannot open input file");
    FILE *output = fopen(argv[2],"w");
    if (!output) error("Cannot open output file");

    cm = new codemanager;
    sl = new symbollist;
    ll = new labellist;
    cl = new constlist;
    strgl = new strglist;
    printf("pass one\n");
    resetparser(); yyparse();
    printf("pass two\n");
    ll->assertalldefined();
    cl->fillin();
    cm->writeout(output);
    printf("%d Bytes of code written\n",cm->len());
    strgl->debugprint();
    cl->debugprint();
    ll->debugprint();
    sl->debugprint();
    delete strgl;
    delete cl;
    delete ll;
    delete sl;
    delete cm;

    if (fclose(output)) error ("cannot close output file");
    if (fclose(yyin)) error ("cannot close input file");
  }catch(const char *const x) {
    fprintf(stderr,"Error:\n%s\n\n",x);
    return 1;
  }catch(const string &x) {
    fprintf(stderr,"Error:\n%s\n\n",x.c_str());
    return 1;
  }catch(...) {
    fprintf(stderr,"Unknown exception caught...\n");
    return 1;
  };
  return 0;
};
