%{
#include <stdlib.h>
#include <stdio.h>
#include "../common/binarydefs.hpp"
#include "global.hpp"

void resetparser ()
{
  currentaddress=0;
};

void writeconst8(const uint8 const8)
{
  cm->put8(currentaddress, const8);
  currentaddress++;
};

void writeconst16(const uint16 const16)
{
  cm->put16(currentaddress, const16);
  currentaddress+=2;
};

void writeconst32(const uint32 const32)
{
  cm->put32(currentaddress, const32);
  currentaddress+=4;
};

inline void writeins(const uint16 ins)
{
  if (currentaddress&1) yyerror("code not 16bit aligned");
  writeconst16(ins);
};

void writestring (const string &s)
{
  for (unsigned i=0;i<s.size();i++)
    cm->put8(currentaddress++, s[i]);
};

%}

%union{
  uint8 regist;
  labellist::index_t labelindex;
  strglist::index_t strgindex;
  constlist::index_t constindex;
  constlist::elem::value_t constnormal;
}

%token COMMAND_NOP
%token COMMAND_PUSHCW
%token COMMAND_PUSHCD
%token COMMAND_JMPCW
%token COMMAND_JMPCD
%token COMMAND_CALLCW
%token COMMAND_CALLCD
%token COMMAND_RET
%token COMMAND_IRET
%token COMMAND_HALT
%token COMMAND_CONSW
%token COMMAND_CONSD
%token COMMAND_PUSHR
%token COMMAND_POPR
%token COMMAND_JMPR
%token COMMAND_CALLR
%token COMMAND_IN
%token COMMAND_OUT
%token COMMAND_INC
%token COMMAND_DEC
%token COMMAND_ADD
%token COMMAND_SUB
%token COMMAND_NEG
%token COMMAND_SMUL
%token COMMAND_SDIV
%token COMMAND_SHL
%token COMMAND_SHR
%token COMMAND_SAR
%token COMMAND_AND
%token COMMAND_OR
%token COMMAND_XOR
%token COMMAND_NOT
%token COMMAND_MOV
%token COMMAND_READCW
%token COMMAND_READCD
%token COMMAND_WRITCW
%token COMMAND_WRITCD
%token COMMAND_XSGCW
%token COMMAND_XSGCD
%token COMMAND_XSGCI
%token COMMAND_XSGECW
%token COMMAND_XSGECD
%token COMMAND_XSGECI
%token COMMAND_XECW
%token COMMAND_XECD
%token COMMAND_XNECW
%token COMMAND_XNECD
%token COMMAND_SHLC
%token COMMAND_SHRC
%token COMMAND_SARC
%token COMMAND_CALLCT
%token COMMAND_JMPCT
%token COMMAND_PUSHCT
%token COMMAND_READR
%token COMMAND_WRITR
%token COMMAND_CONSB

%token ASM_db
%token ASM_dw
%token ASM_dd

%token <regist> REGIST

%token <constnormal> CONSTNORMAL

%token <labelindex> LABELINDEX
%token <strgindex> STRGINDEX

%token ENDOFLINE
%token COMMA
%token COLON
%token PLUS
%token MINUS
%token MULTIPLY
%token DIVIDE
%token BRACKETOPEN
%token BRACKETCLOSE

%type <constindex> exp;
%type <constindex> top5;
%type <constindex> top8;
%type <constindex> top12;
%type <constindex> top16;
%type <constindex> top32;

%left MINUS PLUS
%left MULTIPLY DIVIDE
%left NEG

%%

program: line
	| program line
	;

line: command ENDOFLINE
	| ENDOFLINE
	;

exp: CONSTNORMAL { $$ = cl->addVAL($1); };
	| LABELINDEX { $$ = cl->addLAB($1); }
	| exp PLUS exp { $$ = cl->addADD($1,$3); }
	| exp MINUS exp { $$ = cl->addSUB($1,$3); }
	| exp MULTIPLY exp { $$ = cl->addMUL($1,$3); }
	| exp DIVIDE exp { $$ = cl->addDIV($1,$3); }
	| MINUS exp  %prec NEG { $$ = cl->addSUB(cl->addVAL(0),$2); }
	| BRACKETOPEN exp BRACKETCLOSE { $$ = $2; }
	;

top5: exp { $$ = cl->addTOP5($1); }
	;

top8: exp { $$ = cl->addTOP8($1); }
	;

top12: exp { $$ = cl->addTOP12($1); }
	;

top16: exp { $$ = cl->addTOP16($1); }
	;

top32: exp { $$ = cl->addTOP32($1); }
	;

command: COMMAND_NOP {writeins(0x0000);}
	| COMMAND_PUSHCW top16 {writeins(0x0001);writeconst16(0);cl->get($2).address=currentaddress-2;}
	| COMMAND_PUSHCD top32 {writeins(0x0002);writeconst32(0);cl->get($2).address=currentaddress-4;}
	| COMMAND_JMPCW top16 {writeins(0x0003);writeconst16(0);cl->get($2).address=currentaddress-2;}
	| COMMAND_JMPCD top32 {writeins(0x0004);writeconst32(0);cl->get($2).address=currentaddress-4;}
	| COMMAND_CALLCW top16 {writeins(0x0005);writeconst16(0);cl->get($2).address=currentaddress-2;}
	| COMMAND_CALLCD top32 {writeins(0x0006);writeconst32(0);cl->get($2).address=currentaddress-4;}
	| COMMAND_RET {writeins(0x0007);}
	| COMMAND_IRET {writeins(0x0008);}
	| COMMAND_HALT {writeins(0x0009);}
	| COMMAND_CONSW REGIST COMMA top16 { writeins(uint16($2)+0x0010);writeconst16(0); cl->get($4).address=currentaddress-2; }
	| COMMAND_CONSD REGIST COMMA top32 { writeins(uint16($2)+0x0020);writeconst32(0); cl->get($4).address=currentaddress-4; }
	| COMMAND_PUSHR REGIST {writeins(uint16($2)+0x0030);}
	| COMMAND_POPR REGIST {writeins(uint16($2)+0x0040);}
	| COMMAND_JMPR REGIST {writeins(uint16($2)+0x0050);}
	| COMMAND_CALLR REGIST {writeins(uint16($2)+0x0060);}
	| COMMAND_IN REGIST COMMA REGIST
		{writeins((uint16($2)<<4)+uint16($4)+0x0100);}
	| COMMAND_OUT REGIST COMMA REGIST
		{writeins((uint16($2)<<4)+uint16($4)+0x0200);}
	| COMMAND_INC REGIST COMMA REGIST
		{writeins((uint16($2)<<4)+uint16($4)+0x0300);}
	| COMMAND_DEC REGIST COMMA REGIST
		{writeins((uint16($2)<<4)+uint16($4)+0x0400);}
	| COMMAND_ADD REGIST COMMA REGIST
		{writeins((uint16($2)<<4)+uint16($4)+0x0500);}
	| COMMAND_SUB REGIST COMMA REGIST
		{writeins((uint16($2)<<4)+uint16($4)+0x0600);}
	| COMMAND_NEG REGIST COMMA REGIST
		{writeins((uint16($2)<<4)+uint16($4)+0x0700);}
	| COMMAND_SMUL REGIST COMMA REGIST
		{writeins((uint16($2)<<4)+uint16($4)+0x0800);}
	| COMMAND_SDIV REGIST COMMA REGIST
		{writeins((uint16($2)<<4)+uint16($4)+0x0900);}
	| COMMAND_SHL REGIST COMMA REGIST
		{writeins((uint16($2)<<4)+uint16($4)+0x0A00);}
	| COMMAND_SHR REGIST COMMA REGIST
		{writeins((uint16($2)<<4)+uint16($4)+0x0B00);}
	| COMMAND_SAR REGIST COMMA REGIST
		{writeins((uint16($2)<<4)+uint16($4)+0x0C00);}
	| COMMAND_AND REGIST COMMA REGIST
		{writeins((uint16($2)<<4)+uint16($4)+0x0D00);}
	| COMMAND_OR REGIST COMMA REGIST
		{writeins((uint16($2)<<4)+uint16($4)+0x0E00);}
	| COMMAND_XOR REGIST COMMA REGIST
		{writeins((uint16($2)<<4)+uint16($4)+0x0F00);}
	| COMMAND_NOT REGIST COMMA REGIST
		{writeins((uint16($2)<<4)+uint16($4)+0x1000);}
	| COMMAND_MOV REGIST COMMA REGIST
		{writeins((uint16($2)<<4)+uint16($4)+0x1100);}

	| COMMAND_READCW REGIST COMMA REGIST COMMA top16
		{writeins((uint16($2)<<4)+uint16($4)+0x1200);writeconst16(0);cl->get($6).address=currentaddress-2;}
	| COMMAND_READCD REGIST COMMA REGIST COMMA top32
		{writeins((uint16($2)<<4)+uint16($4)+0x1300);writeconst32(0);cl->get($6).address=currentaddress-4;}

	| COMMAND_WRITCW REGIST COMMA REGIST COMMA top16
		{writeins((uint16($2)<<4)+uint16($4)+0x1400);writeconst16(0);cl->get($6).address=currentaddress-2;}
	| COMMAND_WRITCD REGIST COMMA REGIST COMMA top32
		{writeins((uint16($2)<<4)+uint16($4)+0x1500);writeconst32(0);cl->get($6).address=currentaddress-4;}

	| COMMAND_XSGCW REGIST COMMA REGIST COMMA top16
		{writeins((uint16($2)<<4)+uint16($4)+0x1600); writeconst16(0); cl->get($6).address=currentaddress-2;}
	| COMMAND_XSGCD REGIST COMMA REGIST COMMA top32
		{writeins((uint16($2)<<4)+uint16($4)+0x1700);writeconst32(0); cl->get($6).address=currentaddress-4;}

	| COMMAND_XSGECW REGIST COMMA REGIST COMMA top16
		{writeins((uint16($2)<<4)+uint16($4)+0x1800);writeconst16(0); cl->get($6).address=currentaddress-2;}
	| COMMAND_XSGECD REGIST COMMA REGIST COMMA top32
		{writeins((uint16($2)<<4)+uint16($4)+0x1900);writeconst32(0); cl->get($6).address=currentaddress-4;}

	| COMMAND_XECW REGIST COMMA REGIST COMMA top16
		{writeins((uint16($2)<<4)+uint16($4)+0x1A00);writeconst16(0); cl->get($6).address=currentaddress-2;}
	| COMMAND_XECD REGIST COMMA REGIST COMMA top32
		{writeins((uint16($2)<<4)+uint16($4)+0x1B00);writeconst32(0); cl->get($6).address=currentaddress-4;}

	| COMMAND_XNECW REGIST COMMA REGIST COMMA top16
		{writeins((uint16($2)<<4)+uint16($4)+0x1C00);writeconst16(0); cl->get($6).address=currentaddress-2;}
	| COMMAND_XNECD REGIST COMMA REGIST COMMA top32
		{writeins((uint16($2)<<4)+uint16($4)+0x1D00);writeconst32(0); cl->get($6).address=currentaddress-4;}

	| COMMAND_SHLC REGIST COMMA top5
		{writeins(uint16($2)+0x2000); cl->get($4).address=currentaddress-2; cl->get($4).leftshift=4;}
	| COMMAND_SHRC REGIST COMMA top5
		{writeins(uint16($2)+0x2200); cl->get($4).address=currentaddress-2; cl->get($4).leftshift=4;}
	| COMMAND_SARC REGIST COMMA top5
		{writeins(uint16($2)+0x2400); cl->get($4).address=currentaddress-2; cl->get($4).leftshift=4;}
	| COMMAND_CALLCT top12
		{writeins(0xA000); cl->get($2).address=currentaddress-2; cl->get($2).leftshift=0;}
	| COMMAND_JMPCT top12
		{writeins(0xB000); cl->get($2).address=currentaddress-2; cl->get($2).leftshift=0;}
	| COMMAND_PUSHCT top12
		{writeins(0xC000); cl->get($2).address=currentaddress-2; cl->get($2).leftshift=0;}
	| COMMAND_READR REGIST COMMA REGIST COMMA REGIST
		{writeins((uint16($2)<<8)+(uint16($4)<<4)+uint16($6)+0xD000);}
	| COMMAND_WRITR REGIST COMMA REGIST COMMA REGIST
		{writeins((uint16($2)<<8)+(uint16($4)<<4)+uint16($6)+0xE000);}
	| COMMAND_CONSB REGIST COMMA top8 { writeins((uint16($2)<<8)+0xF000); cl->get($4).address=currentaddress-1; }

	| data
	;

data:	ASM_db dblist data
	| ASM_db dblist
	| ASM_dw dwlist data
	| ASM_dw dwlist
	| ASM_dd ddlist data
	| ASM_dd ddlist
	;

dblist:	dbenc COMMA dblist
	| dbenc
	| strenc COMMA dblist
	| strenc
	;

dwlist:	dwenc COMMA dwlist
	| dwenc
	;

ddlist:	ddenc COMMA ddlist
	| ddenc
	;

strenc:	STRGINDEX { writestring(strgl->get($1).strg); }
	;

dbenc:	top8 { cl->get($1).address=currentaddress; writeconst8(0); }
	;

dwenc:	top16 { cl->get($1).address=currentaddress; writeconst16(0); }
	;

ddenc:	top32 { cl->get($1).address=currentaddress; writeconst32(0); }
	;
